@@ -18,27 +18,10 @@
 # SOFTWARE.
 # ===================================================================
 
-"""SHA-1 cryptographic hash algorithm.
-
-SHA-1_ produces the 160 bit digest of a message.
-
-    >>> from Crypto.Hash import SHA1
-    >>>
-    >>> h = SHA1.new()
-    >>> h.update(b'Hello')
-    >>> print h.hexdigest()
-
-*SHA* stands for Secure Hash Algorithm.
-
-This algorithm is not considered secure. Do not use it for new designs.
-
-.. _SHA-1: http://csrc.nist.gov/publications/fips/fips180-2/fips180-2.pdf
-"""
+from Crypto.Util.py3compat import *
 
 __all__ = ['new', 'block_size', 'digest_size']
 
-from Crypto.Util.py3compat import *
-
 def __make_constructor():
     try:
         # The sha module is deprecated in Python 2.6, so use hashlib when possible.
@@ -79,8 +62,8 @@ def __make_constructor():
 new = __make_constructor()
 del __make_constructor
 
-#: The size of the resulting hash in bytes.
+# The size of the resulting hash in bytes.
 digest_size = new().digest_size
 
-#: The internal block size of the hash algorithm in bytes.
+# The internal block size of the hash algorithm in bytes.
 block_size = new().block_size
