@@ -28,53 +28,6 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ===================================================================
 
-"""Elliptic Curve Cryptography (ECC) algorithms.
-
-ECC_ is a modern and efficient type of public key cryptography.
-Its security is based on the difficulty to solve discrete logarithms
-on the field defined by specific equations involving points on a curve.
-
-ECC can be used to perform signing/verification and asymmetric
-encryption/decryption.
-
-The main benefit of ECC is that the size of a key is significantly smaller
-than with other, more traditional algorithms like RSA or DSA.
-
-For instance, consider the security level equivalent to AES128: an RSA
-key of similar strength must have a modulus of 3072 bits (therefore the total size
-is 768 bytes, comprising modulus and private exponent).
-An ECC private needs as little as 256 bits (32 bytes).
-
-This module provides mechanisms for generating new ECC keys, exporting them
-using widely supported formats like PEM or DER and importing them back.
-
-**This module currently supports only ECC keys defined over the standard
-NIST P-256 curve** (see `FIPS 186-4`_, Section D.1.2.3). More curves will be
-added in the future.
-
-The following example demonstrates how to generate a new key, export it,
-and subsequentely reload it back into the application:
-
-    >>> from Crypto.PublicKey import ECC
-    >>>
-    >>> key = ECC.generate(curve='P-256')
-    >>> f = open('myprivatekey.pem','wt')
-    >>> f.write(key.export_key('PEM'))
-    >>> f.close()
-    ...
-    >>> f = open('myprivatekey.pem','rt')
-    >>> key = RSA.import_key(f.read())
-
-The ECC key can be used to perform or verify ECDSA signatures, see
-`Crypto.Signature.DSS`.
-
-.. _ECC: http://andrea.corbellini.name/2015/05/17/elliptic-curve-cryptography-a-gentle-introduction/
-.. _`FIPS 186-4`: http://nvlpubs.nist.gov/nistpubs/FIPS/NIST.FIPS.186-4.pdf
-
-:undocumented: __package__
-"""
-
-
 import struct
 import binascii
 
@@ -107,7 +60,11 @@ _curve.oid = "1.2.840.10045.3.1.7"
 class EccPoint(object):
     """A class to abstract a point over an Elliptic Curve.
 
-    :undocumented: __init__, __eq__, __neg__, __iadd__, __add__, __mul__
+    :ivar x: The X-coordinate of the ECC point
+    :vartype x: integer
+
+    :ivar y: The Y-coordinate of the ECC point
+    :vartype y: integer
     """
 
     def __init__(self, x, y):
@@ -145,20 +102,22 @@ class EccPoint(object):
 
     @property
     def x(self):
-        """The X-coordinate of the ECC point"""
         if self.is_point_at_infinity():
             raise ValueError("Point at infinity")
         return self._x
 
     @property
     def y(self):
-        """The Y-coordinate of the ECC point"""
         if self.is_point_at_infinity():
             raise ValueError("Point at infinity")
         return self._y
 
     def double(self):
-        """Double this point"""
+        """Double this point (in-place operation).
+
+        :Return:
+            :class:`EccPoint` : this same object (to enable chaining)
+        """
 
         if not self._y:
             return self.point_at_infinity()
@@ -284,16 +243,23 @@ _curve.G = EccPoint(_curve.Gx, _curve.Gy)
 
 
 class EccKey(object):
-    """A private or public key over an Elliptic Curve.
+    r"""Class defining an ECC key.
+    Do not instantiate directly.
+    Use :func:`generate`, :func:`construct` or :func:`import_key` instead.
 
-    :undocumented: __eq__, __repr__, __init__
+    :ivar curve: The name of the ECC curve
+    :vartype curve: string
+
+    :ivar pointQ: an ECC point representating the public component
+    :vartype pointQ: :class:`EccPoint`
+
+    :ivar q: A scalar representating the private component
+    :vartype q: integer
     """
 
     def __init__(self, **kwargs):
         """Create a new ECC key
 
-        Do not instantiate this object directly.
-
         Keywords:
           curve : string
             It must be *"P-256"*, *"prime256v1"* or *"secp256r1"*.
@@ -337,7 +303,8 @@ class EccKey(object):
                (self.pointQ.x, self.pointQ.y, extra)
 
     def has_private(self):
-        """True if this key can be used for making signatures or decrypting"""
+        """``True`` if this key can be used for making signatures or decrypting data."""
+
         return self._d is not None
 
     def _sign(self, z, k):
@@ -361,20 +328,23 @@ class EccKey(object):
 
     @property
     def d(self):
-        """An integer (scalar), representating the private component"""
         if not self.has_private():
             raise ValueError("This is not a private ECC key")
         return self._d
 
     @property
     def pointQ(self):
-        """An `EccPoint`, representating the public component"""
         if self._point is None:
             self._point = _curve.G * self._d
         return self._point
 
     def public_key(self):
-        """Create a new `EccKey`, by retaining only the public components"""
+        """A matching ECC public key.
+
+        Returns:
+            a new :class:`EccKey` object
+        """
+
         return EccKey(curve="P-256", point=self.pointQ)
 
     def _export_subjectPublicKeyInfo(self):
@@ -465,9 +435,8 @@ class EccKey(object):
     def export_key(self, **kwargs):
         """Export this ECC key.
 
-        :Keywords:
-
-          format : string
+        Args:
+          format (string):
             The format to use for wrapping the key:
 
             - *'DER'*. The key will be encoded in an ASN.1 DER_ structure (binary).
@@ -475,29 +444,29 @@ class EccKey(object):
             - *'OpenSSH'*. The key will be encoded in the OpenSSH_ format
               (ASCII, public keys only).
 
-          passphrase : byte string or string
+          passphrase (byte string or string):
             The passphrase to use for protecting the private key.
-            *If not provided, the private key will remain in clear form!*
 
-          use_pkcs8 : boolean
-            In case of a private key, whether the PKCS#8_ representation
-            should be (internally) used. By default it will.
+          use_pkcs8 (boolean):
+            If ``True`` (default and recommended), the `PKCS#8`_ representation
+            will be used.
 
-            Not using PKCS#8 when exporting a private key in
-            password-protected PEM_ form means that the much weaker and
-            unflexible `PEM encryption`_ mechanism will be used.
-            PKCS#8 is therefore always recommended.
+            If ``False``, the much weaker and `PEM encryption`_ mechanism will be used.
 
-          protection : string
-            In case of a private key being exported with password-protection
+          protection (string):
+            When a private key is exported with password-protection
             and PKCS#8 (both ``DER`` and ``PEM`` formats), this parameter MUST be
-            present and be a valid algorithm supported by `Crypto.IO.PKCS8`.
+            present and be a valid algorithm supported by :mod:`Crypto.IO.PKCS8`.
             It is recommended to use ``PBKDF2WithHMAC-SHA1AndAES128-CBC``.
 
-        :Note:
-            In case of a private key being exported with password-protection
-            and PKCS#8_ (both ``DER`` and ``PEM`` formats), all additional parameters
-            will be passed to `Crypto.IO.PKCS8`.
+        .. warning::
+            If you don't provide a passphrase, the private key will be
+            exported in the clear!
+
+        .. note::
+            When exporting a private key with password-protection and `PKCS#8`_
+            (both ``DER`` and ``PEM`` formats), any extra parameters
+            is passed to :mod:`Crypto.IO.PKCS8`.
 
         .. _DER:        http://www.ietf.org/rfc/rfc5915.txt
         .. _PEM:        http://www.ietf.org/rfc/rfc1421.txt
@@ -505,7 +474,8 @@ class EccKey(object):
         .. _`PKCS#8`:   http://www.ietf.org/rfc/rfc5208.txt
         .. _OpenSSH:    http://www.openssh.com/txt/rfc5656.txt
 
-        :Return: A multi-line string (for PEM and OpenSSH) or bytes (for DER) with the encoded key.
+        Returns:
+            A multi-line string (for PEM and OpenSSH) or bytes (for DER) with the encoded key.
         """
 
         args = kwargs.copy()
@@ -552,12 +522,14 @@ class EccKey(object):
 def generate(**kwargs):
     """Generate a new private key on the given curve.
 
-    :Keywords:
-      curve : string
+    Args:
+
+      curve (string):
         Mandatory. It must be "P-256", "prime256v1" or "secp256r1".
-      randfunc : callable
+
+      randfunc (callable):
         Optional. The RNG to read randomness from.
-        If ``None``, the system source is used.
+        If ``None``, :func:`Crypto.Random.get_random_bytes` is used.
     """
 
     curve = kwargs.pop("curve")
@@ -576,15 +548,22 @@ def construct(**kwargs):
     """Build a new ECC key (private or public) starting
     from some base components.
 
-    :Keywords:
-      curve : string
+    Args:
+
+      curve (string):
         Mandatory. It must be "P-256", "prime256v1" or "secp256r1".
-      d : integer
+
+      d (integer):
         Only for a private key. It must be in the range ``[1..order-1]``.
-      point_x : integer
+
+      point_x (integer):
         Mandatory for a public key. X coordinate (affine) of the ECC point.
-      point_y : integer
+
+      point_y (integer):
         Mandatory for a public key. Y coordinate (affine) of the ECC point.
+
+    Returns:
+      :class:`EccKey` : a new ECC key object
     """
 
     point_x = kwargs.pop("point_x", None)
@@ -764,17 +743,17 @@ def _import_openssh(encoded):
 def import_key(encoded, passphrase=None):
     """Import an ECC key (public or private).
 
-    :Parameters:
-      encoded : bytes or a (multi-line) string
+    Args:
+      encoded (bytes or multi-line string):
         The ECC key to import.
 
-        An ECC public key can be:
+        An ECC **public** key can be:
 
         - An X.509 certificate, binary (DER) or ASCII (PEM)
         - An X.509 ``subjectPublicKeyInfo``, binary (DER) or ASCII (PEM)
         - An OpenSSH line (e.g. the content of ``~/.ssh/id_ecdsa``, ASCII)
 
-        An ECC private key can be:
+        An ECC **private** key can be:
 
         - In binary format (DER, see section 3 of `RFC5915`_ or `PKCS#8`_)
         - In ASCII format (PEM or OpenSSH)
@@ -783,16 +762,16 @@ def import_key(encoded, passphrase=None):
 
         For details about the PEM encoding, see `RFC1421`_/`RFC1423`_.
 
-    :Keywords:
-      passphrase : byte string
+      passphrase (byte string):
         The passphrase to use for decrypting a private key.
         Encryption may be applied protected at the PEM level or at the PKCS#8 level.
         This parameter is ignored if the key in input is not encrypted.
 
-    :Return: An ECC key object (`EccKey`)
+    Returns:
+      :class:`EccKey` : a new ECC key object
 
-    :Raise ValueError:
-        When the given key cannot be parsed (possibly because
+    Raises:
+      ValueError: when the given key cannot be parsed (possibly because
         the pass phrase is wrong).
 
     .. _RFC1421: http://www.ietf.org/rfc/rfc1421.txt
