@@ -24,10 +24,11 @@
 
 """Self-testing for PyCrypto hash modules"""
 
+import sys
 import unittest
 import binascii
 import Crypto.Hash
-from Crypto.Util.py3compat import *
+from Crypto.Util.py3compat import b, tobytes
 from Crypto.Util.strxor import strxor_c
 
 class HashDigestSizeSelfTest(unittest.TestCase):
@@ -102,15 +103,6 @@ class HashTestOID(unittest.TestCase):
         h = self.hashmod.new()
         self.assertEqual(h.oid, self.oid)
 
-class HashDocStringTest(unittest.TestCase):
-    def __init__(self, hashmod):
-        unittest.TestCase.__init__(self)
-        self.hashmod = hashmod
-
-    def runTest(self):
-        docstring = self.hashmod.__doc__
-        self.assert_(hasattr(self.hashmod, '__doc__'))
-        self.assert_(isinstance(self.hashmod.__doc__, str))
 
 class GenericHashConstructorTest(unittest.TestCase):
     def __init__(self, hashmod):
@@ -202,7 +194,7 @@ def make_hash_tests(module, module_name, test_data, digest_size, oid=None):
     tests = []
     for i in range(len(test_data)):
         row = test_data[i]
-        (expected, input) = map(b,row[0:2])
+        (expected, input) = map(tobytes,row[0:2])
         if len(row) < 3:
             description = repr(input)
         else:
@@ -214,7 +206,6 @@ def make_hash_tests(module, module_name, test_data, digest_size, oid=None):
 
     if oid is not None:
         tests.append(HashTestOID(module, oid))
-    tests.append(HashDocStringTest(module))
 
     if getattr(module, 'name', None) is not None:
         tests.append(GenericHashConstructorTest(module))
