@@ -33,16 +33,13 @@
 
 """Self-tests for Crypto.Util.asn1"""
 
-__revision__ = "$Id$"
-
 import unittest
 
 from Crypto.Util.py3compat import *
-from Crypto.Util.asn1 import DerObject, DerSetOf, newDerSetOf, DerInteger,\
-                             DerBitString, newDerBitString, newDerObjectId,\
-                             DerObjectId, DerNull, DerOctetString,\
-                             newDerOctetString, DerSequence, newDerSequence,\
-                             newDerInteger
+from Crypto.Util.asn1 import (DerObject, DerSetOf, DerInteger,
+                             DerBitString,
+                             DerObjectId, DerNull, DerOctetString,
+                             DerSequence)
 
 class DerObjectTests(unittest.TestCase):
 
@@ -92,6 +89,12 @@ class DerObjectTests(unittest.TestCase):
         der.payload = b('ppll')
         self.assertEquals(der.encode(), b('\x9E\x04ppll'))
 
+    def testObjEncode5(self):
+        # Encode type with explicit tag
+        der = DerObject(0x10, explicit=5)
+        der.payload = b("xxll")
+        self.assertEqual(der.encode(), b("\xa5\x06\x10\x04xxll"))
+
     # -----
 
     def testObjDecode1(self):
@@ -99,14 +102,14 @@ class DerObjectTests(unittest.TestCase):
         der = DerObject(0x02)
         der.decode(b('\x02\x02\x01\x02'))
         self.assertEquals(der.payload, b("\x01\x02"))
-        self.assertEquals(der._idOctet, 0x02)
+        self.assertEquals(der._tag_octet, 0x02)
 
     def testObjDecode2(self):
         # Decode long payload
         der = DerObject(0x02)
         der.decode(b('\x02\x81\x80' + "1"*128))
         self.assertEquals(der.payload, b("1")*128)
-        self.assertEquals(der._idOctet, 0x02)
+        self.assertEquals(der._tag_octet, 0x02)
 
     def testObjDecode3(self):
         # Decode payload with too much data gives error
@@ -114,7 +117,7 @@ class DerObjectTests(unittest.TestCase):
         self.assertRaises(ValueError, der.decode, b('\x02\x02\x01\x02\xFF'))
         # Decode payload with too little data gives error
         der = DerObject(0x02)
-        self.assertRaises(EOFError, der.decode, b('\x02\x02\x01'))
+        self.assertRaises(ValueError, der.decode, b('\x02\x02\x01'))
 
     def testObjDecode4(self):
         # Decode implicit tag (primitive)
@@ -137,13 +140,31 @@ class DerObjectTests(unittest.TestCase):
         # Arbitrary DER object
         der = DerObject()
         der.decode(b('\x65\x01\x88'))
-        self.assertEquals(der._idOctet, 0x65)
+        self.assertEquals(der._tag_octet, 0x65)
         self.assertEquals(der.payload, b('\x88'))
 
+    def testObjDecode7(self):
+        # Decode explicit tag
+        der = DerObject(0x10, explicit=5)
+        der.decode(b("\xa5\x06\x10\x04xxll"))
+        self.assertEquals(der._tag_octet, 0x10)
+        self.assertEquals(der.payload, b('xxll'))
+
+        # Explicit tag may be 0
+        der = DerObject(0x10, explicit=0)
+        der.decode(b("\xa0\x06\x10\x04xxll"))
+        self.assertEquals(der._tag_octet, 0x10)
+        self.assertEquals(der.payload, b('xxll'))
+
+    def testObjDecode8(self):
+        # Verify that decode returns the object
+        der = DerObject(0x02)
+        self.assertEqual(der, der.decode(b('\x02\x02\x01\x02')))
+
 class DerIntegerTests(unittest.TestCase):
 
     def testInit1(self):
-        der = newDerInteger(1)
+        der = DerInteger(1)
         self.assertEquals(der.encode(), b('\x02\x01\x01'))
 
     def testEncode1(self):
@@ -201,6 +222,11 @@ class DerIntegerTests(unittest.TestCase):
         der = DerInteger(-87873)
         self.assertEquals(der.encode(), b('\x02\x03\xFE\xA8\xBF'))
 
+    def testEncode4(self):
+        # Explicit encoding
+        number = DerInteger(0x34, explicit=3)
+        self.assertEquals(number.encode(), b('\xa3\x03\x02\x01\x34'))
+
     # -----
 
     def testDecode1(self):
@@ -268,15 +294,29 @@ class DerIntegerTests(unittest.TestCase):
         der.decode(b('\x02\x00'))
         self.assertEquals(der.value, 0)
 
+    def testDecode6(self):
+        # Explicit encoding
+        number = DerInteger(explicit=3)
+        number.decode(b('\xa3\x03\x02\x01\x34'))
+        self.assertEquals(number.value, 0x34)
+
+    def testDecode7(self):
+        # Verify decode returns the DerInteger
+        der = DerInteger()
+        self.assertEquals(der, der.decode(b('\x02\x01\x7F')))
+
+    ###
+
     def testErrDecode1(self):
         # Wide length field
         der = DerInteger()
         self.assertRaises(ValueError, der.decode, b('\x02\x81\x01\x01'))
 
+
 class DerSequenceTests(unittest.TestCase):
 
     def testInit1(self):
-        der = newDerSequence(1, DerInteger(2), '0\x00')
+        der = DerSequence([1, DerInteger(2), b('0\x00')])
         self.assertEquals(der.encode(), b('0\x08\x02\x01\x01\x02\x01\x020\x00'))
 
     def testEncode1(self):
@@ -372,11 +412,19 @@ class DerSequenceTests(unittest.TestCase):
         self.assertEquals(der.encode(), b('0\x09\x02\x01\x02\x02\x01\x09\x02\x01\x08'))
 
     def testEncode7(self):
-        # One integer and another type (no matter what it is)
+        # One integer and another type (already encoded)
         der = DerSequence()
         der.append(0x180L)
-        der.append(b('\x00\x02\x00\x00'))
-        self.assertEquals(der.encode(), b('0\x08\x02\x02\x01\x80\x00\x02\x00\x00'))
+        der.append(b('0\x03\x02\x01\x05'))
+        self.assertEquals(der.encode(), b('0\x09\x02\x02\x01\x800\x03\x02\x01\x05'))
+        self.failIf(der.hasOnlyInts())
+
+    def testEncode8(self):
+        # One integer and another type (yet to encode)
+        der = DerSequence()
+        der.append(0x180L)
+        der.append(DerSequence([5]))
+        self.assertEquals(der.encode(), b('0\x09\x02\x02\x01\x800\x03\x02\x01\x05'))
         self.failIf(der.hasOnlyInts())
 
     ####
@@ -457,12 +505,19 @@ class DerSequenceTests(unittest.TestCase):
         self.failIf(der.hasOnlyInts())
         self.failIf(der.hasOnlyInts(False))
 
+    def testDecode9(self):
+        # Verify that decode returns itself
+        der = DerSequence()
+        self.assertEqual(der, der.decode(b('0\x06\x24\x02\xb6\x63\x12\x00')))
+
+    ###
+
     def testErrDecode1(self):
         # Not a sequence
         der = DerSequence()
-        self.assertRaises(EOFError, der.decode, b(''))
+        self.assertRaises(ValueError, der.decode, b(''))
         self.assertRaises(ValueError, der.decode, b('\x00'))
-        self.assertRaises(EOFError, der.decode, b('\x30'))
+        self.assertRaises(ValueError, der.decode, b('\x30'))
 
     def testErrDecode2(self):
         der = DerSequence()
@@ -473,15 +528,34 @@ class DerSequenceTests(unittest.TestCase):
         # Wrong length format
         der = DerSequence()
         # Missing length in sub-item
-        self.assertRaises(EOFError, der.decode, b('\x30\x04\x02\x01\x01\x00'))
+        self.assertRaises(ValueError, der.decode, b('\x30\x04\x02\x01\x01\x00'))
         # Valid BER, but invalid DER length
         self.assertRaises(ValueError, der.decode, b('\x30\x81\x03\x02\x01\x01'))
         self.assertRaises(ValueError, der.decode, b('\x30\x04\x02\x81\x01\x01'))
 
+    def test_expected_nr_elements(self):
+        der_bin = DerSequence([1, 2, 3]).encode()
+
+        DerSequence().decode(der_bin, nr_elements=3)
+        DerSequence().decode(der_bin, nr_elements=(2,3))
+        self.assertRaises(ValueError, DerSequence().decode, der_bin, nr_elements=1)
+        self.assertRaises(ValueError, DerSequence().decode, der_bin, nr_elements=(4,5))
+
+    def test_expected_only_integers(self):
+
+        der_bin1 = DerSequence([1, 2, 3]).encode()
+        der_bin2 = DerSequence([1, 2, DerSequence([3, 4])]).encode()
+
+        DerSequence().decode(der_bin1, only_ints_expected=True)
+        DerSequence().decode(der_bin1, only_ints_expected=False)
+        DerSequence().decode(der_bin2, only_ints_expected=False)
+        self.assertRaises(ValueError, DerSequence().decode, der_bin2, only_ints_expected=True)
+
+
 class DerOctetStringTests(unittest.TestCase):
 
     def testInit1(self):
-        der = newDerOctetString(b('\xFF'))
+        der = DerOctetString(b('\xFF'))
         self.assertEquals(der.encode(), b('\x04\x01\xFF'))
 
     def testEncode1(self):
@@ -503,6 +577,11 @@ class DerOctetStringTests(unittest.TestCase):
         der.decode(b('\x04\x02\x01\x02'))
         self.assertEquals(der.payload, b('\x01\x02'))
 
+    def testDecode2(self):
+        # Verify that decode returns the object
+        der = DerOctetString()
+        self.assertEqual(der, der.decode(b('\x04\x00')))
+
     def testErrDecode1(self):
         # No leftovers allowed
         der = DerOctetString()
@@ -519,12 +598,12 @@ class DerNullTests(unittest.TestCase):
     def testDecode1(self):
         # Empty sequence
         der = DerNull()
-        der.decode(b('\x05\x00'))
+        self.assertEquals(der, der.decode(b('\x05\x00')))
 
 class DerObjectIdTests(unittest.TestCase):
 
     def testInit1(self):
-        der = newDerObjectId("1.1")
+        der = DerObjectId("1.1")
         self.assertEquals(der.encode(), b('\x06\x01)'))
 
     def testEncode1(self):
@@ -543,12 +622,23 @@ class DerObjectIdTests(unittest.TestCase):
         der.decode(b('\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01'))
         self.assertEquals(der.value, '1.2.840.113549.1.1.1')
 
+    def testDecode2(self):
+        # Verify that decode returns the object
+        der = DerObjectId()
+        self.assertEquals(der,
+                der.decode(b('\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01')))
+
+
 class DerBitStringTests(unittest.TestCase):
 
     def testInit1(self):
-        der = newDerBitString(b("\xFF"))
+        der = DerBitString(b("\xFF"))
         self.assertEquals(der.encode(), b('\x03\x02\x00\xFF'))
 
+    def testInit2(self):
+        der = DerBitString(DerInteger(1))
+        self.assertEquals(der.encode(), b('\x03\x04\x00\x02\x01\x01'))
+
     def testEncode1(self):
         # Empty sequence
         der = DerBitString()
@@ -572,10 +662,16 @@ class DerBitStringTests(unittest.TestCase):
         der.decode(b('\x03\x03\x00\x01\x02'))
         self.assertEquals(der.value, b('\x01\x02'))
 
+    def testDecode2(self):
+        # Verify that decode returns the object
+        der = DerBitString()
+        self.assertEquals(der, der.decode(b('\x03\x00')))
+
+
 class DerSetOfTests(unittest.TestCase):
 
     def testInit1(self):
-        der = newDerSetOf(DerInteger(1), DerInteger(2))
+        der = DerSetOf([DerInteger(1), DerInteger(2)])
         self.assertEquals(der.encode(), b('1\x06\x02\x01\x01\x02\x01\x02'))
 
     def testEncode1(self):
@@ -639,6 +735,14 @@ class DerSetOfTests(unittest.TestCase):
         self.assertRaises(ValueError, der.decode,
             b('0\x0A\x02\x02\x01\x80\x24\x02\xb6\x63\x12\x00'))
 
+    def testDecode4(self):
+        # Verify that decode returns the object
+        der = DerSetOf()
+        self.assertEquals(der,
+                der.decode(b('1\x08\x02\x02\x01\x80\x02\x02\x00\xff')))
+
+    ###
+
     def testErrDecode1(self):
         # No leftovers allowed
         der = DerSetOf()
